from collections.abc import Callable

import structlog
from sqlalchemy import BinaryExpression

from orchestrator.db import ProductBlockTable, ProductTable
from orchestrator.db.filters import QueryType, generic_filter
from orchestrator.db.filters.generic_filters import (
    generic_is_like_filter,
    generic_values_in_column_filter,
)
from orchestrator.db.filters.search_filters import default_inferred_column_clauses, node_to_str_val
from orchestrator.db.filters.search_filters.inferred_filter import filter_exact
from orchestrator.utils.search_query import Node

logger = structlog.get_logger(__name__)


def product_block_filter(query: QueryType, value: str) -> QueryType:
    """Filter ProductBlocks by '-'-separated list of Product block 'name' (column) values."""
    blocks = value.split("-")
    return query.filter(ProductTable.product_blocks.any(ProductBlockTable.name.in_(blocks)))


def product_block_clause(node: Node) -> BinaryExpression:
    return ProductTable.product_blocks.any(ProductBlockTable.name.ilike(node_to_str_val(node)))


PRODUCT_FILTER_FUNCTIONS_BY_COLUMN: dict[str, Callable[[QueryType, str], QueryType]] = {
    "product_id": generic_is_like_filter(ProductTable.product_id),
    "name": generic_is_like_filter(ProductTable.name),
    "description": generic_is_like_filter(ProductTable.description),
    "product_type": generic_is_like_filter(ProductTable.product_type),
    "status": generic_values_in_column_filter(ProductTable.status),
    "tag": generic_values_in_column_filter(ProductTable.tag),
    "product_blocks": product_block_filter,
}

PRODUCT_TABLE_COLUMN_CLAUSES = default_inferred_column_clauses(ProductTable) | {
    "productBlock": product_block_clause,
    "product_block": product_block_clause,
    "tag": filter_exact(ProductTable.tag),
}

product_filter_fields = list(PRODUCT_FILTER_FUNCTIONS_BY_COLUMN.keys())
filter_products = generic_filter(PRODUCT_FILTER_FUNCTIONS_BY_COLUMN)



from sqlalchemy.dialects import postgresql
from sqlalchemy.sql.elements import CompilerElement


def to_sql_string(stmt: CompilerElement) -> str:
    dialect = postgresql.dialect()  # type: ignore
    return str(stmt.compile(dialect=dialect, compile_kwargs={"literal_binds": True}))



import pytest
from pydantic import TypeAdapter, ValidationError

from orchestrator.forms.validators import MTU
from test.unit_tests.forms.test_generic_validators import stringify_exceptions

mtu_adapter = TypeAdapter(MTU)


@pytest.mark.parametrize("mtu", (1500, 9000))
def test_mtu(mtu):
    mtu_adapter.validate_python(mtu)


def test_mtu_schema():
    assert mtu_adapter.json_schema() == {"maximum": 9000, "minimum": 1500, "multipleOf": 7500, "type": "integer"}


def test_mtu_nok_ge():
    with pytest.raises(ValidationError) as error_info:
        mtu_adapter.validate_python(1499)

    expected = [
        {
            "input": 1499,
            "ctx": {"ge": 1500},
            "loc": (),
            "msg": "Input should be greater than or equal to 1500",
            "type": "greater_than_equal",
        }
    ]
    assert error_info.value.errors(include_url=False) == expected


def test_mtu_nok_le():
    with pytest.raises(ValidationError) as error_info:
        mtu_adapter.validate_python(9001)

    expected = [
        {
            "input": 9001,
            "ctx": {"le": 9000},
            "loc": (),
            "msg": "Input should be less than or equal to 9000",
            "type": "less_than_equal",
        },
    ]
    assert error_info.value.errors(include_url=False) == expected


def test_mtu_nok_1500_or_9000():
    with pytest.raises(ValidationError) as error_info:
        mtu_adapter.validate_python(1600)

    expected = [
        {
            "input": 1600,
            "ctx": {"error": ValueError("MTU must be either 1500 or 9000")},
            "loc": (),
            "msg": "Value error, MTU must be either 1500 or 9000",
            "type": "value_error",
        },
    ]
    assert stringify_exceptions(error_info.value.errors(include_url=False)) == stringify_exceptions(expected)



# Copyright 2019-2023 SURF.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from collections.abc import Collection, Generator, Iterable
from contextvars import ContextVar
from enum import StrEnum, auto
from http import HTTPStatus
from typing import Any

import structlog
from graphql import GraphQLError
from httpx import HTTPStatusError, Response
from strawberry.extensions import SchemaExtension
from strawberry.types import ExecutionContext, Info

logger = structlog.stdlib.get_logger(__name__)

EXTENSION_ERROR_TYPE = "error_type"
EXTENSION_HTTP_STATUS_CODE = "http_status_code"

_error_bucket: ContextVar[None | list[GraphQLError]] = ContextVar("error_bucket", default=None)


class ErrorType(StrEnum):
    """These error types are returned in 'error_type' extension field on an error response.

    https://productionreadygraphql.com/2020-08-01-guide-to-graphql-errors
    https://engineering.zalando.com/posts/2021/04/modeling-errors-in-graphql.html

    We currently distinguish 3 categories that are meaningful to the frontend/user:
     - NOT_AUTHORIZED: user not allowed to access, may be solved by renewing token or a different entitlement
     - NOT_FOUND: a resource wasn't found, data inconsistency or wrong identifier
     - INTERNAL_ERROR: all other errors on the graphql server and/or backend systems. User may retry later
    """

    NOT_AUTHORIZED = auto()
    NOT_FOUND = auto()
    INTERNAL_ERROR = auto()


def _is_http_error(exception: Exception, *statuses: HTTPStatus) -> bool:
    match exception:
        case HTTPStatusError(response=Response(status_code=s)) if s in statuses:
            return True

    return False


def _to_error_type(exception: Exception | None) -> ErrorType:
    # For https://git.ia.surfsara.nl/netdev/automation/projects/orchestrator/-/issues/1892 when we move this extension
    # to a library, we could make it possible to initialize the ErrorHandlerExtension with an additional (custom)
    # ErrorType mapper. Because each GraphQL project will have different exceptions
    match exception:
        case PermissionError():
            return ErrorType.NOT_AUTHORIZED
        case Exception() if _is_http_error(exception, HTTPStatus.FORBIDDEN, HTTPStatus.UNAUTHORIZED):
            return ErrorType.NOT_AUTHORIZED
        case Exception() if _is_http_error(exception, HTTPStatus.NOT_FOUND):
            return ErrorType.NOT_FOUND
        case _:
            return ErrorType.INTERNAL_ERROR


def _get_all_errors(execution_context: ExecutionContext) -> Iterable[GraphQLError]:
    # Errors from strawberry or exceptions raised by our code
    if execution_context.result and execution_context.result.errors:
        yield from execution_context.result.errors

    # Errors registered by our code
    if registered_errors := _error_bucket.get():
        yield from registered_errors


def _has_extension(error: GraphQLError, key: str) -> bool:
    return key in (error.extensions or {})


def _add_extension(error: GraphQLError, key: str, value: Any) -> None:
    if error.extensions is None:
        error.extensions = {}
    error.extensions[key] = value


def _process(error: GraphQLError) -> GraphQLError:
    exc = error.original_error
    if isinstance(exc, HTTPStatusError):
        _add_extension(error, EXTENSION_HTTP_STATUS_CODE, {f"{exc.request.url}": exc.response.status_code})
    if not _has_extension(error, EXTENSION_ERROR_TYPE):
        _add_extension(error, EXTENSION_ERROR_TYPE, str(_to_error_type(exc)))
    return error


class ErrorHandlerExtension(SchemaExtension):
    """Collect all raised and/or registered errors and enriches them with metadata.

    Disambiguation:
    - This class is a Strawberry extension that is executed on each graphql query, like a middleware.
    - The metadata added to each error is also called an 'extension' because it is a reserved field in the GraphQL spec.
    More details: https://spec.graphql.org/October2021/#sec-Errors.Error-result-format
    """

    def on_execute(self) -> Generator[None, None, None]:
        _error_bucket.set([])

        yield

        if not self.execution_context or self.execution_context.result is None:
            return

        self.execution_context.result.errors = [_process(error) for error in _get_all_errors(self.execution_context)]


def _register(message: str, path: Collection[str], error_type: ErrorType) -> None:
    error = GraphQLError(message=message, path=path, extensions={EXTENSION_ERROR_TYPE: str(error_type)})
    if (errors := _error_bucket.get()) is None:
        logger.debug("ErrorHandlerExtension disabled, dropping error", error=error)
        return

    logger.debug("Registering error", error=error)
    errors.append(error)
    _error_bucket.set(errors)


def register_error(message: str, info: Info, error_type: ErrorType = ErrorType.INTERNAL_ERROR) -> None:
    """Register an error message.

    Use this to collect error messages from multiple resolvers and return them in the response.
    """
    _register(message, info.path, error_type)


def register_exception(exception: Exception, info: Info) -> None:
    """Register an exception encountered during the query execution.

    Use this to collect exceptions from multiple resolvers and return them in the response.
    """
    _register(str(exception), info.path, _to_error_type(exception))



# Copyright 2019-2020 SURF.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import annotations

import warnings

from nwastdlib.vlans import VlanRanges

__all__ = [
    "VlanRanges",
]

warnings.warn(
    "VlanRanges will be removed from orchestrator-core in an upcoming release. "
    "Please import it from nwastdlib (>= 1.5.0) instead.",
    DeprecationWarning,
    stacklevel=1,
)



# Copyright 2019-2023 SURF.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from collections.abc import Callable

from sqlalchemy import ColumnClause

from orchestrator.db.filters.filters import QueryType


def generic_bool_filter(field: ColumnClause) -> Callable[[QueryType, str], QueryType]:
    def bool_filter(query: QueryType, value: str) -> QueryType:
        value_as_bool = value.lower() in ("yes", "y", "true", "1")
        return query.filter(field.is_(value_as_bool))

    return bool_filter



# Copyright 2019-2020 SURF.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from typing import Annotated, Any

from annotated_types import Ge, Le
from pydantic import AfterValidator, BaseModel, ConfigDict, Field, model_serializer, model_validator


class BFD(BaseModel):
    model_config = ConfigDict(json_schema_extra={"format": "optGroup"})

    # order matters, this should be first
    enabled: bool
    minimum_interval: Annotated[int, Field(ge=1, le=255000)] | None = 900
    multiplier: Annotated[int, Field(ge=1, le=255)] | None = 3

    @model_validator(mode="after")
    def check_optional_fields(self) -> "BFD":
        if not self.enabled:
            self.minimum_interval = None
            self.multiplier = None

        return self

    @model_serializer
    def bfd_serializer(self) -> dict[str, Any]:
        if not self.enabled:
            # If BFD is disabled the interval and multiplier are None. We need to exclude them from
            # the output to prevent overriding their default values in the form
            return {"enabled": self.enabled}

        return {"enabled": self.enabled, "minimum_interval": self.minimum_interval, "multiplier": self.multiplier}


MTU_LOW = 1500
MTU_HIGH = 9000
MTU_MULTIPLIER = 7500


def validate_mtu(val: int) -> int:
    if val not in (MTU_LOW, MTU_HIGH):
        raise ValueError(f"MTU must be either {MTU_LOW} or {MTU_HIGH}")
    return val


MTU = Annotated[
    int,
    Ge(MTU_LOW),
    Le(MTU_HIGH),
    AfterValidator(validate_mtu),
    Field(json_schema_extra={"multipleOf": MTU_MULTIPLIER}),
]



from http import HTTPStatus
from os import getenv

import pytest
from redis import Redis
from sqlalchemy import func, select

from orchestrator.db import WorkflowTable, db
from orchestrator.domain.base import SubscriptionModel
from orchestrator.services.workflows import get_workflows
from orchestrator.settings import app_settings
from orchestrator.targets import Target
from orchestrator.utils.functional import orig
from orchestrator.workflows.steps import cache_domain_models
from test.unit_tests.fixtures.workflows import add_soft_deleted_workflows  # noqa: F401

PRODUCT_ID = "fb28e465-87fd-4d23-9c75-ed036529e416"


@pytest.fixture(autouse=True)
def _add_soft_deleted_workflows(add_soft_deleted_workflows):  # noqa: F811
    add_soft_deleted_workflows(10)


def test_workflows(test_client):
    response = test_client.get("/api/workflows")

    assert response.status_code == HTTPStatus.OK
    workflows = response.json()

    assert len(workflows) == len(list(get_workflows()))
    assert all(workflow["name"] is not None for workflow in workflows)
    assert all(workflow["target"] is not None for workflow in workflows)


def test_deleted_workflows_are_filtered(test_client):
    all_workflows = get_workflows(include_deleted=True)
    response = test_client.get("/api/workflows")

    assert response.status_code == HTTPStatus.OK
    workflows = response.json()
    assert len(workflows) == len(list(all_workflows)) - 10


@pytest.mark.parametrize("target", (Target.CREATE, Target.TERMINATE, Target.MODIFY))
def test_workflows_by_target(target, test_client):
    response = test_client.get(f"/api/workflows?target={target}")
    workflows = response.json()

    num_wfs = db.session.scalar(select(func.count(WorkflowTable.workflow_id)).where(WorkflowTable.target == target))
    assert len(workflows) == num_wfs
    assert all(target == workflow["target"] for workflow in workflows)


@pytest.mark.parametrize(
    "include_steps, predicate",
    ((False, lambda workflow: workflow["steps"] is None), (True, lambda workflow: len(workflow["steps"]) > 0)),
)
def test_workflows_include_steps(include_steps, predicate, test_client):
    response = test_client.get(f"/api/workflows?include_steps={include_steps}")

    assert response.status_code == HTTPStatus.OK
    workflows = response.json()

    assert len(workflows) == len(list(get_workflows()))
    assert all(predicate(workflow) for workflow in workflows)


def test_get_all_with_product_tags(test_client):
    response = test_client.get("/api/workflows/with_product_tags")

    assert response.status_code == HTTPStatus.OK
    num_wfs = len(db.session.scalars(WorkflowTable.select()).all())
    assert len(response.json()) == num_wfs


@pytest.mark.skipif(
    not getenv("AIOCACHE_DISABLE", "0") == "0", reason="AIOCACHE must be enabled for this test to do anything"
)
def test_push_subscriptions_to_cache_step(generic_subscription_1):
    push_subscription = orig(cache_domain_models)
    push_subscription("Aribtrary_name", SubscriptionModel.from_subscription(generic_subscription_1))

    cache = Redis.from_url(str(app_settings.CACHE_URI))
    assert cache.get(f"domain:{generic_subscription_1}") is None

    app_settings.CACHE_DOMAIN_MODELS = True
    push_subscription("Aribtrary_name", SubscriptionModel.from_subscription(generic_subscription_1))
    result = cache.get(f"domain:{generic_subscription_1}")
    assert result is not None
    app_settings.CACHE_DOMAIN_MODELS = False



from collections.abc import Callable

import structlog
from sqlalchemy import BinaryExpression, select
from sqlalchemy.inspection import inspect

from orchestrator.db import ProductTable, WorkflowTable
from orchestrator.db.filters import QueryType, generic_filter
from orchestrator.db.filters.generic_filters import (
    generic_is_like_filter,
    generic_range_filters,
)
from orchestrator.db.filters.search_filters import default_inferred_column_clauses, inferred_filter
from orchestrator.db.filters.search_filters.inferred_filter import filter_exact
from orchestrator.utils.helpers import to_camel
from orchestrator.utils.search_query import Node, WhereCondGenerator

logger = structlog.get_logger(__name__)

created_at_range_filters = generic_range_filters(WorkflowTable.created_at)


def products_filter(query: QueryType, value: str) -> QueryType:
    """Filter ProductBlocks by '-'-separated list of Product 'name' (column) values."""
    products = value.split("-")
    return query.filter(WorkflowTable.products.any(ProductTable.name.in_(products)))


def make_product_clause(filter_generator: WhereCondGenerator) -> WhereCondGenerator:
    """The passed filter_generator takes a Node and returns a where clause acting on a ProductTable column."""

    def product_clause(node: Node) -> BinaryExpression:
        subq = select(WorkflowTable.workflow_id).join(WorkflowTable.products).where(filter_generator(node)).subquery()
        return WorkflowTable.workflow_id.in_(subq)

    return product_clause


BASE_CAMEL = {to_camel(key): generic_is_like_filter(value) for key, value in inspect(WorkflowTable).columns.items()}

WORKFLOW_FILTER_FUNCTIONS_BY_COLUMN: dict[str, Callable[[QueryType, str], QueryType]] = (
    BASE_CAMEL | {"products": products_filter} | created_at_range_filters
)


WORKFLOW_TABLE_COLUMN_CLAUSES = default_inferred_column_clauses(WorkflowTable) | {
    "product": make_product_clause(inferred_filter(ProductTable.name)),
    "tag": make_product_clause(filter_exact(ProductTable.tag)),
}


workflow_filter_fields = list(WORKFLOW_FILTER_FUNCTIONS_BY_COLUMN.keys())
filter_workflows = generic_filter(WORKFLOW_FILTER_FUNCTIONS_BY_COLUMN)



from collections.abc import Callable

import structlog
from sqlalchemy import BinaryExpression, inspect
from sqlalchemy.orm import MappedColumn

from orchestrator.db import ProductBlockTable, ResourceTypeTable
from orchestrator.db.filters import QueryType, generic_filter
from orchestrator.db.filters.generic_filters import generic_is_like_filter
from orchestrator.db.filters.search_filters import default_inferred_column_clauses, node_to_str_val
from orchestrator.utils.helpers import to_camel
from orchestrator.utils.search_query import Node

logger = structlog.get_logger(__name__)


def product_blocks_filter(query: QueryType, value: str) -> QueryType:
    """Filter ResourceTypes by '-'-separated list of Product block 'name' (column) values."""
    product_blocks = value.split("-")
    return query.filter(ResourceTypeTable.product_blocks.any(ProductBlockTable.name.in_(product_blocks)))


def product_blocks_clause(node: Node) -> BinaryExpression:
    return ResourceTypeTable.product_blocks.any(ProductBlockTable.name.ilike(node_to_str_val(node)))


RESOURCE_TYPE_FILTER_FUNCTIONS_BY_COLUMN: dict[str, Callable[[QueryType, str], QueryType]] = {
    "resourceTypeId": generic_is_like_filter(ResourceTypeTable.resource_type_id),
    "resourceType": generic_is_like_filter(ResourceTypeTable.resource_type),
    "description": generic_is_like_filter(ResourceTypeTable.description),
    "productBlocks": product_blocks_filter,
}

RESOURCE_TYPE_TABLE_COLUMN_MAPPINGS: dict[str, MappedColumn] = {
    k: column for key, column in inspect(ResourceTypeTable).columns.items() for k in [key, to_camel(key)]
} | {"product_block": ProductBlockTable.name, "productBlock": ProductBlockTable.name}
resource_type_filter_fields = list(RESOURCE_TYPE_FILTER_FUNCTIONS_BY_COLUMN.keys())
filter_resource_types = generic_filter(RESOURCE_TYPE_FILTER_FUNCTIONS_BY_COLUMN)

RESOURCE_TYPE_TABLE_COLUMN_CLAUSES = default_inferred_column_clauses(ResourceTypeTable) | {
    "product_block": product_blocks_clause,
    "productBlock": product_blocks_clause,
}



import pytest
from pydantic import ValidationError

from orchestrator.forms.validators import BFD
from pydantic_forms.core import FormPage


def test_bfd_ok():
    class Form(FormPage):
        bfd: BFD

    model = {"enabled": False, "minimum_interval": None, "multiplier": None}
    validated_data = Form(bfd=model).model_dump()

    expected = {"bfd": BFD(enabled=False, minimum_interval=None, multiplier=None).model_dump()}
    assert expected == validated_data

    model = {"enabled": True, "minimum_interval": 600, "multiplier": 3}
    validated_data = Form(bfd=model).model_dump()

    expected = {"bfd": BFD(enabled=True, minimum_interval=600, multiplier=3).model_dump()}
    assert expected == validated_data


def test_bfd_schema():
    class Form(FormPage):
        bfd: BFD

    assert Form.model_json_schema() == {
        "additionalProperties": False,
        "$defs": {
            "BFD": {
                "format": "optGroup",
                "properties": {
                    "enabled": {"title": "Enabled", "type": "boolean"},
                    "minimum_interval": {
                        "anyOf": [{"maximum": 255000, "minimum": 1, "type": "integer"}, {"type": "null"}],
                        "default": 900,
                        "title": "Minimum " "Interval",
                    },
                    "multiplier": {
                        "anyOf": [{"maximum": 255, "minimum": 1, "type": "integer"}, {"type": "null"}],
                        "default": 3,
                        "title": "Multiplier",
                    },
                },
                "required": ["enabled"],
                "title": "BFD",
                "type": "object",
            }
        },
        "properties": {"bfd": {"$ref": "#/$defs/BFD"}},
        "required": ["bfd"],
        "title": "unknown",
        "type": "object",
    }


def test_bfd_enabled_missing_values():
    class Form(FormPage):
        bfd: BFD

    model = {"enabled": True, "multiplier": 4}
    assert Form(bfd=model).model_dump() == {"bfd": {"enabled": True, "multiplier": 4, "minimum_interval": 900}}

    model = {"enabled": True, "minimum_interval": 600}
    assert Form(bfd=model).model_dump() == {"bfd": {"enabled": True, "multiplier": 3, "minimum_interval": 600}}


def test_bfd_disabled_missing_values():
    class Form(FormPage):
        bfd: BFD

    model = {"enabled": False}
    assert Form(bfd=model).model_dump() == {"bfd": {"enabled": False}}

    model = {"enabled": False, "minimum_interval": 600, "multiplier": 3}
    assert Form(bfd=model).model_dump() == {"bfd": {"enabled": False}}


def test_bfd_wrong_minimum_interval_ge():
    class Form(FormPage):
        bfd: BFD

    with pytest.raises(ValidationError) as error_info:
        Form(bfd={"enabled": True, "minimum_interval": 0, "multiplier": 3})

    expected = [
        {
            "input": 0,
            "loc": ("bfd", "minimum_interval"),
            "msg": "Input should be greater than or equal to 1",
            "type": "greater_than_equal",
            "ctx": {"ge": 1},
        },
    ]
    assert error_info.value.errors(include_url=False) == expected


def test_bfd_wrong_minimum_interval_le():
    class Form(FormPage):
        bfd: BFD

    with pytest.raises(ValidationError) as error_info:
        Form(bfd={"enabled": True, "minimum_interval": 255001, "multiplier": 3})

    expected = [
        {
            "input": 255001,
            "loc": ("bfd", "minimum_interval"),
            "msg": "Input should be less than or equal to 255000",
            "type": "less_than_equal",
            "ctx": {"le": 255000},
        },
    ]
    assert error_info.value.errors(include_url=False) == expected


def test_bfd_wrong_multiplier_ge():
    class Form(FormPage):
        bfd: BFD

    with pytest.raises(ValidationError) as error_info:
        Form(bfd={"enabled": True, "minimum_interval": 600, "multiplier": 0})

    expected = [
        {
            "input": 0,
            "loc": ("bfd", "multiplier"),
            "msg": "Input should be greater than or equal to 1",
            "type": "greater_than_equal",
            "ctx": {"ge": 1},
        },
    ]
    assert error_info.value.errors(include_url=False) == expected


def test_bfd_wrong_multiplier_le():
    class Form(FormPage):
        bfd: BFD

    with pytest.raises(ValidationError) as error_info:
        Form(bfd={"enabled": True, "minimum_interval": 600, "multiplier": 256})

    expected = [
        {
            "input": 256,
            "loc": ("bfd", "multiplier"),
            "msg": "Input should be less than or equal to 255",
            "type": "less_than_equal",
            "ctx": {"le": 255},
        },
    ]
    assert error_info.value.errors(include_url=False) == expected



# Copyright 2019-2020 SURF.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import structlog

from orchestrator.forms.validators.network_type_validators import BFD, MTU

# TODO: Deprecated, file will be deleted in the future
logger = structlog.get_logger(__name__)
logger.warn("DEPRECATED import: BFD and MTU validators are moved to 'orchestrator.forms.validators'")
__all__ = ["BFD", "MTU"]
